#ifndef __resummation__
#define __resummation__

#include "Vlumifns.hh"
#include "math/complex.hh"

namespace resumm {

  enum {
    LL=0,
    NLL=1,
    NNLL=2
  };
  enum {
    LO=0,
    NLO=1,
    NNLO=2
  };

  // set the kind of logs are used by the Borel prescription
  // (see M.Bonvini, S.Forte, G.Ridolfi, NPB xxxxx, arXiv:1009.5691 [hep-ph]):
  // type = 0  ->  eq. (3.23)
  // type = 1  ->  eq. (3.29)
  // type = 2  ->  eq. (3.21)  -- currently not completely tested
  extern int SetBorelType(int type);

  // choose to use the luminosity by Becher, Neubert, Xu
  // eq. (4.44) of M.Bonvini, S.Forte, G.Ridolfi NPB xxxxx, arXiv:1009.5691 [hep-ph]
  // only applicable to rapidity distributions
  extern int SetBNXluminosity(bool);


  // Initializations
  extern int InitFromVtot();
  extern int InitFromVrap();
  extern int Init(double S, double alpha_s, double muF_q_ratio, double muR_q_ratio, int n_f, collider coll);

  extern int DontUseApproxPDFsForBorelPrescription(bool use=true);
  extern int DontUseApproxPDFsForMinimalPrescription(bool use=true);


  extern double Borel_tot(double x, double C, unsigned int log_order=1, unsigned int match_order=1,
			  unsigned int N_cheb=10);
  extern double Minimal_tot(double x, unsigned int log_order=1, unsigned int match_order=1,
			    unsigned int N_cheb=15, int method=1, double beta=7., double gamma=3.);
  extern double Borel_rap(double Y, double x, double C, unsigned int log_order=1, unsigned int match_order=1,
			  unsigned int N_cheb=15);
  extern double Minimal_rap(double Y, double x, unsigned int log_order=1, unsigned int match_order=1,
			    unsigned int N_cheb=15, int method=1, double beta=7., double gamma=3.);

};




#endif
